@@ -11,6 +11,10 @@ from inplaceeditform.fields import (AdaptorForeingKeyField,
 
 class AdaptorAutoCompleteProvider(object):
 
+    def __init__(self, *args, **kwargs):
+        super(AdaptorAutoCompleteProvider, self).__init__(*args, **kwargs)
+        self.config['can_auto_save'] = 0
+
     def install_ajax_select(self):
         if 'ajax_select' in settings.INSTALLED_APPS:
             lookup = self.config.get('lookup', None)
@@ -37,7 +41,7 @@ class AdaptorAutoCompleteProvider(object):
     def render_value_edit(self):
         value = super(AdaptorAutoCompleteProvider, self).render_value_edit()
         if self.install_ajax_select():
-            return render_to_string('inplaceeditform_extra_fields/adaptor_autocomplete/render_value.html',
+            return render_to_string('inplaceeditform_extra_fields/adaptor_autocomplete/render_value_edit.html',
                                     {'value': value,
                                     'STATIC_URL': get_static_url(),
                                     'STATIC_URL_AJAX_SELECTS': get_static_url('ajax_selects'),
@@ -111,82 +115,116 @@ class AdaptorTinyMCEField(AdaptorTextAreaField):
     def name(self):
         return 'tiny'
 
+    @property
+    def classes(self):
+        return super(AdaptorTinyMCEField, self).classes + " textareainplaceedit"
+
     def __init__(self, *args, **kwargs):
         super(AdaptorTinyMCEField, self).__init__(*args, **kwargs)
         self.widget_options = self.config and self.config.get('widget_options', {})
-
-    def install_cmsutils(self):
-        if 'cmsutils' in settings.INSTALLED_APPS:
-            try:
-                from cmsutils.forms.widgets import TinyMCE
-                return True
-            except ImportError:
-                pass
-        return False
+        self.config['can_auto_save'] = 0
 
     @property
     def TinyMCE(self):
-        from cmsutils.forms.widgets import TinyMCE
+        from inplaceeditform_extra_fields.widgets import TinyMCE
         return TinyMCE
 
+    def treatment_height(self, height, width=None):
+        height = super(AdaptorTinyMCEField, self).treatment_height(height, width=width)
+        if isinstance(height, basestring) and height.endswith('px'):
+            height = height[:-2]
+        return height
+
+    def treatment_width(self, width, height=None):
+        height = super(AdaptorTinyMCEField, self).treatment_width(width, height=width)
+        if isinstance(height, basestring) and width.endswith('px'):
+            width = height[:-2]
+        return width
+
     def get_field(self):
         field = super(AdaptorTinyMCEField, self).get_field()
-        if not self.install_cmsutils():
-            return field
+        inplace_edit_auto_save = getattr(settings, 'INPLACEEDIT_AUTO_SAVE', False)
         tiny_mce_buttons = {
-            '0': ['bold', 'italic', 'underline', 'justifyleft',
-                  'justifycenter', 'justifyright', 'justifyfull'],
-            '1': ['bullist', 'numlist', 'outdent', 'indent'],
-            '2': ['undo', 'redo'],
-            '3': ['cut', 'copy', 'paste', 'pasteword'],
-            '4': ['forecolor', 'link', 'code', 'internal_links'],
-            '5': ['iframes', 'image', 'file', 'removeformat'],
-            }
-        tiny_mce_selectors = {'0': ['fontsizeselect'],
-                              '1': ['formatselect', 'fontselect'],
-                              '2': ['styleselect']}
-
-        extra_mce_settings = getattr(settings, 'EXTRA_MCE', {})
+            '0': ['apply_inplace_edit', 'cancel_inplace_edit'],
+            '1': ['undo', 'redo'],
+            '2': ['bold', 'italic', 'underline', 'justifyleft',
+                'justifycenter', 'justifyright', 'justifyfull'],
+            '3': ['bullist', 'numlist', 'outdent', 'indent'],
+        }
+        if not inplace_edit_auto_save:
+            tiny_mce_buttons['4'] = ['cut', 'copy', 'paste', 'pasteword']
+            tiny_mce_buttons['5'] = ['forecolor', 'link', 'code', 'internal_links']
+            tiny_mce_buttons['6'] = ['iframes', 'file', 'removeformat']
+            tiny_mce_selectors = {'0': ['fontsizeselect'],
+                                  '1': ['formatselect', 'fontselect'],
+                                  '2': ['styleselect']}
+        else:
+            tiny_mce_selectors = {}
+        extra_mce_settings = {}
         extra_mce_settings.update(self._order_tinymce_buttons(tiny_mce_buttons, tiny_mce_selectors))
-
         tiny_extra_media = getattr(settings, 'TINYMCE_EXTRA_MEDIA', {})
         content_css = [i for i in tiny_extra_media.get('css', [])]
-        content_css = ','.join(["%s%s" % (get_static_url(), css) for css in content_css])
+        css_page = self.config.get('__css', '').split(',')
+        content_css = ["%s%s" % (get_static_url(), css) for css in content_css]
+        content_css += css_page
+        content_css = ','.join(content_css)
+        include_content_css = getattr(settings, 'TINYMCE_INCLUDE_CONTENT_CSS', False)
+        if content_css:
+            js_css = ["ed.dom.loadCSS('%s')" % css for css in content_css.split(',')]
+            load_css = """function loadMyCSS(ed) {
+                %s
+            }""" % ';'.join(js_css)
+            extra_mce_settings['init_instance_callback'] = "loadMyCSS"
+            extra_mce_settings['functions'] = load_css
+        if not content_css or include_content_css:
+            content_css = False
         content_js = [i for i in tiny_extra_media.get('css', [])]
         extra_mce_settings.update({'inplace_edit': True,
+                                   'inplace_edit_auto_save': getattr(settings, 'INPLACEEDIT_AUTO_SAVE', False),
                                    'theme_advanced_blockformats': 'h1,h2,h4,blockquote',
-                                   'file_browser_callback': 'CustomFileBrowser',
-                                   'theme_advanced_statusbar_location': "bottom",
-                                   'theme_advanced_resizing': True,
-                                   'theme_advanced_resize_horizontal': True,
-                                   'convert_on_click': True,
-                                   'content_css': content_css,
+                                   'theme_advanced_statusbar_location': "none",
+                                   'theme_advanced_toolbar_location': "external",
+                                   'theme_advanced_resizing': False,
+                                   'theme_advanced_resize_horizontal': False,
+                                   'content_css': False,
                                    'content_js': content_js})
-        extra_mce_settings.update(self.widget_options)
-        field.field.widget = self.TinyMCE(extra_mce_settings=extra_mce_settings,
-                                             print_head=False)
+        if 'height' in self.widget_options:
+            extra_mce_settings['height'] = self.treatment_height(self.widget_options['height'], self.widget_options.get('width', None))
+            extra_mce_settings['min_height'] = extra_mce_settings['height']
+        if 'width' in self.widget_options:
+            extra_mce_settings['width'] = self.treatment_width(self.widget_options['width'], self.widget_options.get('height', None))
+        extra_mce_settings.update(getattr(settings, 'INPLACE_EXTRA_MCE', {}))
+        field.field.widget = self.TinyMCE(extra_mce_settings=extra_mce_settings)
         return field
 
-    def render_value(self, field_name=None):
+    def _render_value(self, field_name=None):
         return mark_safe(super(AdaptorTinyMCEField, self).render_value(field_name=field_name))
 
+    def render_value(self, field_name=None):
+        value = self._render_value(field_name)
+        classes = ' '.join(self.classes.split(' ')[1:])
+        return render_to_string('inplaceeditform_extra_fields/adaptor_tiny/render_value.html',
+                                {'value': value,
+                                 'classes': classes,
+                                 'adaptor': self})
+
     def render_value_edit(self):
-        value = super(AdaptorTinyMCEField, self).render_value_edit()
-        if self.install_cmsutils():
-            return render_to_string('inplaceeditform_extra_fields/adaptor_tiny/render_value.html',
-                                    {'value': value,
-                                     'adaptor': self,
-                                     'field': self.get_field(),
-                                     'is_ajax': self.request.is_ajax()})
-        return value
+        value = self._render_value()
+        return render_to_string('inplaceeditform_extra_fields/adaptor_tiny/render_value_edit.html',
+                                {'value': value,
+                                 'adaptor': self,
+                                 'field': self.get_field(),
+                                 'is_ajax': self.request.is_ajax()})
+
+    def render_field(self, template_name="inplaceeditform_extra_fields/adaptor_tiny/render_field.html", extra_context=None):
+        return super(AdaptorTinyMCEField, self).render_field(template_name=template_name,
+                                                             extra_context=extra_context)
 
     def render_media_field(self,
                           template_name="inplaceeditform_extra_fields/adaptor_tiny/render_media_field.html",
                           extra_context=None):
-        if self.install_cmsutils():
-            return super(AdaptorTinyMCEField, self).render_media_field(template_name=template_name,
-                                                                       extra_context=extra_context)
-        return super(AdaptorTinyMCEField, self).render_media_field()
+        return super(AdaptorTinyMCEField, self).render_media_field(template_name=template_name,
+                                                                    extra_context=extra_context)
 
     def _order_tinymce_buttons(self, buttons_priorized, selectors_priorized,
                                button_width=20, selector_width=80):
@@ -206,15 +244,10 @@ class AdaptorTinyMCEField(AdaptorTextAreaField):
                                                             selector_width)
         buttons_width = len(buttons) * button_width
         selectors_width = len(selectors) * selector_width
-
         if total_width >= buttons_width + selectors_width:  # one row
-            if total_width >= selectors_width:
-                result['theme_advanced_buttons1'] = ','.join(buttons)
-                result['theme_advanced_buttons2'] = ','.join(selectors)
-            else:
-                num_selectors = (total_width - buttons_width) / selector_width
-                result['theme_advanced_buttons1'] = ','.join(selectors[:num_selectors] + buttons)
-
+            buttons_selectors = buttons + selectors
+            theme_advanced_buttons1 = ','.join(buttons_selectors)
+            result['theme_advanced_buttons1'] = theme_advanced_buttons1
         elif total_width * 2 >= buttons_width + selectors_width:  # two rows
             aux_index = total_width / button_width
             if total_width >= buttons_width:
@@ -241,8 +274,9 @@ class AdaptorTinyMCEField(AdaptorTextAreaField):
         buttons_list = []
         selectors_list = []
         # we assume that we have more priority levels on buttons
-
-        for key in buttons:
+        buttons_keys = buttons.keys()
+        buttons_keys.sort()
+        for key in buttons_keys:
             if (used_width + button_width * len(buttons[key])) < total_width:
                 buttons_list += buttons[key]
                 used_width += button_width * len(buttons[key])
